require "surbase/version"
require "surbase/controller"

require "open3"
require "base64"

module Surbase
  class Command
    def self.execute( info )
      Command.new.execute( info )
    end
    
    attr_accessor   :info, :out, :err, :status
    
    def execute( info )
      if ! info.instance_of?( Hash )
        info = {
          :cmd => info
        }
      else
        return nil if ! info.key?( :cmd )
      end
      
      @out, @err, status = Open3.capture3( info[ :cmd ], { :stdin_data => info[ :stdin_data ] } )
      @status = status.exitstatus
      @info = info
      self
    end
    
    def output
      puts @info[ :cmd ]
      puts @out if ! @out.empty?
      $stderr.puts @err if ! @err.empty?
    end
  end
  
  class Status
    Success         = "0"
    Unknown         = "-1"
    ModelSaveError  = "-100"
    
    @@status = {
        Success         => "Success",
        Unknown         => "Unknown Error",
        ModelSaveError  => "Model Save Error",
    }
    
    def self.msg( code )
      @@status.key?( code ) ? @@status[ code ] : "Unknown Code=#{code}"
    end
    
    def self.set( code, msg )
      @@status[ code ] = msg
      Status.new( code )
    end
    
    attr_accessor   :code, :values
    
    def initialize( code = Unknown, values = {} )
      @code   = code
      @values = values
    end
    
    def msg
      Status.msg( @code )
    end
  end
  
  class Cipher
    def initialize( name )
      @cipher = OpenSSL::Cipher.new( name )
    end
    
    def encode( value )
      Base64.urlsafe_encode64( value )
    end
    
    def decode( encoded_value )
      Base64.urlsafe_decode64( encoded_value )
    end
    
    def encrypt( data, pass )
      @cipher.encrypt
      @cipher.pkcs5_keyivgen( pass )
      encode( @cipher.update( data ) + @cipher.final )
    end
    
    def decrypt( data, pass )
      @cipher.decrypt
      @cipher.pkcs5_keyivgen( pass )
      @cipher.update( decode( data ) ) + @cipher.final
    end
  end
  
  class Secrets
    def self.set( key, value )
      ENV[ key.to_s ] = value.to_s
    end
    
    def self.get( key )
      ENV[ key.to_s ]
    end
    
    def self.setup( &block )
      module_eval( &block )
    end
  end
  
  class Config
    @@prefix = ""
    @@providers = []
    
    def self.prefix( prefix )
      @@prefix = prefix
    end
    
    def self.setup( &block )
      require "#{Rails.root}/config/surbase_secrets.rb"
      
      module_eval( &block )
      
      Rails.application.config.middleware.use OmniAuth::Builder do
        @@providers.each{|provider|
          provider provider[ :provider ], provider[ :key ], provider[ :secret ]
        }
      end
    end
    
    def self.provider( provider )
      @@providers.push({
        :provider => provider,
        :key      => Secrets.get( "#{@@prefix}#{provider.upcase}_KEY" ),
        :secret   => Secrets.get( "#{@@prefix}#{provider.upcase}_SECRET" ),
      })
    end
  end
end
