require "surbase/model"

module Surbase
  module Controller
    def initialize
      super
      
      @cipher = Cipher.new( Secrets.get( :CIPHER_NAME ) )
    end
    
  protected
    def result_hash( status )
      {
        :status => {
          :code     => status.code,
          :msg      => status.msg,
          :values   => status.values,
        }
      }
    end
    
    def result_api( status )
      render :json => result_hash( status )
    end
    
    def cookie( key, value, expires )
      cookies[ key ] = { :value => value, :expires => expires }
    end
    
    def to_json( object )
      JSON.generate( object )
    end
    
    def from_json( json )
      JSON.parse( json )
    end
    
    def create_key( salt )
      Digest::SHA512::hexdigest( salt )
    end
    
    def create_key_random
      now = Time.now.to_i
      create_key( [
        now,
        rand( now ),
        rand( now ),
        rand( now )
      ].join( "_" ) )
    end
    
    def encrypt_json( access_token, pass )
      @cipher.encrypt( to_json( access_token ), pass )
    end
    
    def decrypt_json( encrypted_access_token, pass )
      from_json( @cipher.decrypt( encrypted_access_token, pass ) )
    end
    
    def error_status( code, exception )
      status = Status.new( code, [ exception.message ] )
      status.values.concat( exception.backtrace ) if ! Rails.env.production?
      status
    end
  end
  
  module AccountsController
    include Controller
    
    def login
      redirect_to "/auth/#{params[ :provider ]}"
    end
    
    def auth_callback
      auth = request.env[ "omniauth.auth" ]
      result_api( login_callback({
        :auth         => auth,
        :provider     => auth[ "provider" ],
        :uid          => auth[ "uid" ],
        :provider_id  => %Q(#{auth[ "provider" ]}\t#{auth[ "uid" ]}),
        :name         => auth[ "info" ][ "name" ],
        :icon         => auth[ "info" ][ "image" ],
        :access_token => {
          :token      => auth[ "credentials" ][ "token" ],
          :secret     => auth[ "credentials" ][ "secret" ],
        },
        :login_key    => create_key_random
      }) )
    end
    
  protected
    def login_callback( info )
      status = Status.new
      
      account = nil
      ActiveRecord::Base.transaction do
        account_data = {
          :provider_id => info[ :provider_id ],
          :login_key   => info[ :login_key ],
        }
        account = Model.load( ::Account, [ "provider_id=?", account_data[ :provider_id ] ], account_data )
        account.save!
        
        provider_account_data = {
          :account_id             => account.id,
          :name                   => info[ :name ],
          :icon                   => info[ :icon ],
          :encrypted_access_token => encrypt_json( info[ :access_token ], info[ :login_key ] ),
        }
        provider_account = Model.load( ::ProviderAccount, [ "account_id=?", account.id ], provider_account_data )
        provider_account.save!
      end
        status.code = Status::Success
        status.values = {
          :provider_info => {
            :provider => info[ :provider ],
            :name     => info[ :name ],
            :icon     => info[ :icon ],
          },
          :account_id => @cipher.encrypt( @cipher.encode( account.id.to_s ), info[ :login_key ] ),
          :login_key  => info[ :login_key ],
        }
        status
      rescue => e
        error_status( Status::ModelSaveError, e )
    end
  end
end
